@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemComplexPlugin.cpp 3952 2007-01-29 00:26:24Z hadess $
+ * $Id: totemComplexPlugin.cpp 3968 2007-01-30 10:09:51Z hadess $
  */
 
 #include <mozilla-config.h>
@@ -43,7 +43,7 @@
 #define WARN_UNIMPLEMENTED()\
 static PRBool warned = PR_FALSE;\
 if (!warned) {\
-	D ("WARNING! Use of unimplemented function 'totemIComplexPlayer::%s'", __FUNCTION__);\
+	D ("WARNING! Use of unimplemented function 'totemIComplexPlayer::%s'", G_GNUC_FUNCTION);\
 	warned = PR_TRUE;\
 }
 
@@ -53,7 +53,7 @@ if (!warned) {\
 #define SHOW_CALL()\
 static PRBool called = PR_FALSE;\
 if (!called) {\
-	D ("NOTE! Use of function 'totemIComplexPlayer::%s'", __FUNCTION__);\
+	D ("NOTE! Use of function 'totemIComplexPlayer::%s'", G_GNUC_FUNCTION);\
 	called = PR_TRUE;\
 }
 #else
