@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemMullYPlugin.cpp 3717 2006-11-15 17:21:16Z chpe $
+ * $Id: totemMullYPlugin.cpp 3928 2007-01-22 14:59:07Z chpe $
  */
 
 #include <mozilla-config.h>
@@ -38,6 +38,8 @@
 
 #include "totemIMullYPlayer.h"
 
+#include "totemClassInfo.h"
+
 #include "totemMullYPlugin.h"
 
 /* 67DABFBF-D0AB-41fa-9C46-CC0F21721616 */
@@ -86,91 +88,12 @@ NS_IMPL_ISUPPORTS2 (totemScriptablePlugin,
 
 /* nsIClassInfo */
 
-NS_IMETHODIMP
-totemScriptablePlugin::GetFlags (PRUint32 *aFlags)
-{
-  *aFlags = nsIClassInfo::PLUGIN_OBJECT | nsIClassInfo::DOM_OBJECT;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetImplementationLanguage (PRUint32 *aImplementationLanguage)
-{
-  *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetInterfaces (PRUint32 *count,
-				      nsIID * **array)
-{
-  *array = NS_STATIC_CAST (nsIID**, nsMemory::Alloc (sizeof (nsIID)));
-  if (!*array)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *count = 1;
-
-  (*array)[0] = NS_STATIC_CAST (nsIID*,
-  				nsMemory::Clone (&NS_GET_IID (totemIMullYPlayer),
-						 sizeof(nsIID)));
-  if (!(*array)[0]) {
-    NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY (0, *array);
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  g_message ("GetInterfaces");
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetHelperForLanguage (PRUint32 language,
-					     nsISupports **_retval)
-{
-  *_retval = nsnull;
-  g_message ("GetHelperForLanguage %d", language);
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetContractID (char * *aContractID)
-{
-  *aContractID = nsnull;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassDescription (char * *aClassDescription)
-{
-  *aClassDescription = NS_STATIC_CAST (char*,
-				       nsMemory::Clone (kClassDescription,
-						        sizeof (kClassDescription)));
-  if (!*aClassDescription)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  g_message ("GetClassDescription: %s", *aClassDescription);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassID (nsCID * *aClassID)
-{
-  *aClassID = NS_STATIC_CAST (nsCID*,
-			      nsMemory::Clone (&kClassID,
-					       sizeof (nsCID*)));
-  if (!*aClassID)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  g_message ("GetClassID");
-  return NS_OK;
-}
-     
-NS_IMETHODIMP
-totemScriptablePlugin::GetClassIDNoAlloc (nsCID *aClassIDNoAlloc)
-{
-  *aClassIDNoAlloc = kClassID;
-  g_message ("GetClassIDNoAlloc");
-  return NS_OK;
-}
+TOTEM_CLASSINFO_BEGIN (totemScriptablePlugin,
+		       1,
+		       kClassID,
+		       kClassDescription)
+  TOTEM_CLASSINFO_ENTRY (0, totemIMullYPlayer)
+TOTEM_CLASSINFO_END
 
 /* totemIMullYPlayer */
 
