@@ -38,6 +38,20 @@ enum {
   NUM_SIGNALS
 };
 
+struct BaconVolumeButton {
+  GtkButton parent;
+
+  /* popup */
+  GtkWidget *dock, *scale, *image, *plus, *min;
+  GtkIconSize size;
+  gint click_id;
+  float direction;
+  guint32 pop_time;
+  GdkPixbuf *icon[4];
+  GtkTooltips *tooltips;
+  guint timeout : 1;
+};
+
 static void	bacon_volume_button_class_init	(BaconVolumeButtonClass * klass);
 static void	bacon_volume_button_init	(BaconVolumeButton * button);
 static void	bacon_volume_button_dispose	(GObject        * object);
@@ -134,6 +148,7 @@ bacon_volume_button_init (BaconVolumeButton *button)
   button->timeout = FALSE;
   button->click_id = 0;
   button->dock = button->scale = NULL;
+  button->tooltips = gtk_tooltips_new ();
 }
 
 static void
@@ -576,6 +591,7 @@ cb_dock_key_release (GtkWidget      * widget,
     display = gtk_widget_get_display (widget);
     gdk_display_keyboard_ungrab (display, event->time);
     gdk_display_pointer_ungrab (display, event->time);
+    gtk_grab_remove (button->dock);
 
     /* hide again */
     gtk_widget_hide (button->dock);
@@ -747,6 +763,30 @@ bacon_volume_button_update_icon (BaconVolumeButton *button)
   gtk_image_set_from_pixbuf (GTK_IMAGE (button->image), pixbuf);
 }
 
+static void
+bacon_volume_button_update_tip (BaconVolumeButton *button)
+{
+  GtkRange *range = GTK_RANGE (button->scale);
+  float val = gtk_range_get_value (range);
+  GtkAdjustment *adj;
+  char *str;
+
+  adj = gtk_range_get_adjustment (range);
+
+  if (val == adj->lower) {
+    str = g_strdup (_("Muted"));
+  } else if (val == adj->upper) {
+    str = g_strdup (_("Full Volume"));
+  } else {
+    str = g_strdup_printf ("%d %%",
+			   (int) ((val - adj->lower) / (adj->upper - adj->lower) * 100));
+  }
+
+  gtk_tooltips_set_tip (button->tooltips, GTK_WIDGET (button),
+			str, NULL);
+  g_free (str);
+}
+
 static void
 bacon_volume_scale_value_changed (GtkRange * range)
 {
@@ -754,6 +794,7 @@ bacon_volume_scale_value_changed (GtkRange * range)
   BaconVolumeButton *button = scale->button;
 
   bacon_volume_button_update_icon (button);
+  bacon_volume_button_update_tip (button);
 
   /* signal */
   g_signal_emit (button, signals[SIGNAL_VALUE_CHANGED], 0);
@@ -769,6 +810,8 @@ bacon_volume_button_load_icons (GtkWidget *widget)
   GtkIconTheme *theme;
   const char *icon_name[] = {"audio-volume-muted", "audio-volume-low",
     "audio-volume-medium", "audio-volume-high"};
+  const char *fallback_icon_name[] = {"stock_volume-0", "stock_volume-min",
+    "stock_volume-med", "stock_volume-max"};
 
   screen = gtk_widget_get_screen (widget);
   theme = gtk_icon_theme_get_for_screen (screen);
@@ -784,7 +827,13 @@ bacon_volume_button_load_icons (GtkWidget *widget)
     button->icon[i] = gtk_icon_theme_load_icon (theme, icon_name[i], w, 0, &error);
     if (error) {
       g_print ("Couldn't load themed icon '%s': %s\n", icon_name[i], error->message);
-      g_error_free (error);
+      g_clear_error (&error);
+
+      button->icon[i] = gtk_icon_theme_load_icon (theme, fallback_icon_name[i], w, 0, &error);
+      if (error) {
+	g_print ("Couldn't load themed icon '%s': %s\n", icon_name[i], error->message);
+	g_clear_error (&error);
+      }
     }
   }
 
