@@ -30,12 +30,13 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "data/totem-mime-types.h"
+#include "totem-mime-types.h"
 #include "totem-uri.h"
 #include "totem-private.h"
 
 static GtkFileFilter *filter_all;
 static GtkFileFilter *filter_supported;
+static GtkFileFilter *filter_audio, *filter_video;
 
 gboolean
 totem_playing_dvd (const char *uri)
@@ -159,6 +160,16 @@ totem_uri_get_subtitle_uri (const char *uri)
 	guint len, i;
 	GnomeVFSURI *vfsuri;
 
+	if (g_str_has_prefix (uri, "http") != FALSE) {
+		return NULL;
+	}
+
+	/* Does gnome-vfs support that scheme? */
+	vfsuri = gnome_vfs_uri_new (uri);
+	if (vfsuri == NULL)
+		return NULL;
+	gnome_vfs_uri_unref (vfsuri);
+
 	if (strstr (uri, "#subtitle:") != NULL) {
 		return NULL;
 	}
@@ -179,7 +190,7 @@ totem_uri_get_subtitle_uri (const char *uri)
 				gnome_vfs_uri_unref (vfsuri);
 				return subtitle;
 			}
-		gnome_vfs_uri_unref (vfsuri);
+			gnome_vfs_uri_unref (vfsuri);
 		}
 	}
 	g_free (subtitle);
@@ -222,22 +233,55 @@ totem_setup_file_filters (void)
 	filter_all = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter_all, _("All files"));
 	gtk_file_filter_add_pattern (filter_all, "*");
+	g_object_ref (filter_all);
 
 	filter_supported = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter_supported,
-			_("Supported files"));
+	gtk_file_filter_set_name (filter_supported, _("Supported files"));
 	for (i = 0; i < G_N_ELEMENTS (mime_types); i++) {
 		gtk_file_filter_add_mime_type (filter_supported, mime_types[i]);
 	}
+
+	/* Add the special Disc-as-files formats */
+	gtk_file_filter_add_mime_type (filter_supported, "application/x-cd-image");
+	gtk_file_filter_add_mime_type (filter_supported, "application/x-cue");
+	g_object_ref (filter_supported);
+
+	/* Audio files */
+	filter_audio = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_audio, _("Audio files"));
+	for (i = 0; i < G_N_ELEMENTS (audio_mime_types); i++) {
+		gtk_file_filter_add_mime_type (filter_audio, audio_mime_types[i]);
+	}
+	g_object_ref (filter_audio);
+
+	/* Video files */
+	filter_video = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter_video, _("Video files"));
+	for (i = 0; i < G_N_ELEMENTS (video_mime_types); i++) {
+		gtk_file_filter_add_mime_type (filter_video, video_mime_types[i]);
+	}
+	gtk_file_filter_add_mime_type (filter_video, "application/x-cd-image");
+	gtk_file_filter_add_mime_type (filter_video, "application/x-cue");
+	g_object_ref (filter_video);
+}
+
+void
+totem_destroy_file_filters (void)
+{
+	g_object_unref (filter_all);
+	g_object_unref (filter_supported);
+	g_object_unref (filter_audio);
+	g_object_unref (filter_video);
 }
 
 GSList *
-totem_add_files (GtkWindow *parent, const char *path, char **new_path)
+totem_add_files (GtkWindow *parent, const char *path)
 {
 	GtkWidget *fs;
 	int response;
 	GSList *filenames;
-	char *mrl;
+	char *mrl, *new_path;
+	GConfClient *conf;
 
 	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
 			parent,
@@ -247,14 +291,24 @@ totem_add_files (GtkWindow *parent, const char *path, char **new_path)
 			NULL);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_all);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_supported);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_audio);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fs), filter_video);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fs), filter_supported);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 
+	conf = gconf_client_get_default ();
 	if (path != NULL) {
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
-				path);
+		gtk_file_chooser_set_current_folder_uri
+			(GTK_FILE_CHOOSER (fs), path);
+	} else {
+		new_path = gconf_client_get_string (conf, "/apps/totem/open_path", NULL);
+		if (new_path != NULL && new_path != '\0') {
+			gtk_file_chooser_set_current_folder_uri
+				(GTK_FILE_CHOOSER (fs), new_path);
+		}
+		g_free (new_path);
 	}
 
 	response = gtk_dialog_run (GTK_DIALOG (fs));
@@ -262,21 +316,29 @@ totem_add_files (GtkWindow *parent, const char *path, char **new_path)
 	while (gtk_events_pending())
 		gtk_main_iteration();
 
-	if (response != GTK_RESPONSE_ACCEPT)
+	if (response != GTK_RESPONSE_ACCEPT) {
+		gtk_widget_destroy (fs);
+		g_object_unref (conf);
 		return NULL;
+	}
 
 	filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
 	if (filenames == NULL) {
 		gtk_widget_destroy (fs);
+		g_object_unref (conf);
 		return NULL;
 	}
 
 	mrl = filenames->data;
-	if (mrl != NULL && new_path != NULL) {
-		*new_path = g_path_get_dirname (mrl);
+	if (mrl != NULL) {
+		new_path = g_path_get_dirname (mrl);
+		gconf_client_set_string (conf, "/apps/totem/open_path",
+				new_path, NULL);
+		g_free (new_path);
 	}
 
 	gtk_widget_destroy (fs);
+	g_object_unref (conf);
 
 	return filenames;
 }
