@@ -20,7 +20,7 @@
  * permission are above and beyond the permissions granted by the GPL license
  * Totem is covered by.
  *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * Monday 7th February 2005: Christian Schaller: Add exception clause.
  * See license_change file for details.
  *
  */
@@ -30,8 +30,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <popt.h>
-
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
@@ -43,12 +41,22 @@
 
 /* #define THUMB_DEBUG */
 
+#ifdef G_HAVE_ISO_VARARGS
+#define PROGRESS_DEBUG(...) { if (verbose != FALSE) g_message (__VA_ARGS__); }
+#elif defined(G_HAVE_GNUC_VARARGS)
+#define PROGRESS_DEBUG(format...) { if (verbose != FALSE) g_message (format); }
+#endif
+
 #define MIN_LEN_FOR_SEEK 25000
 #define HALF_SECOND G_USEC_PER_SEC * .5
 #define BORING_IMAGE_VARIANCE 256.0		/* Tweak this if necessary */
 
 gboolean finished = FALSE;
-static char *format = "png";
+static gboolean jpeg_output = FALSE;
+static gboolean output_size = 128;
+static gboolean time_limit = TRUE;
+static gboolean verbose = FALSE;
+static char **filenames = NULL;
 
 #ifdef THUMB_DEBUG
 static void
@@ -80,13 +88,13 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 
 	if (holes == NULL)
 	{
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
 	g_assert (gdk_pixbuf_get_has_alpha (pixbuf) == FALSE);
 	g_assert (gdk_pixbuf_get_has_alpha (holes) != FALSE);
-	target = gdk_pixbuf_ref (pixbuf);
+	target = g_object_ref (pixbuf);
 
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
 	{
@@ -97,7 +105,7 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 	}
 
 	tmp = gdk_pixbuf_flip (holes, FALSE);
-	gdk_pixbuf_unref (holes);
+	g_object_unref (holes);
 	holes = tmp;
 
 	for (i = 0; i < height; i += gdk_pixbuf_get_height (holes))
@@ -110,7 +118,7 @@ add_holes_to_pixbuf_small (GdkPixbuf *pixbuf, int width, int height)
 				1, 1, GDK_INTERP_NEAREST, 255);
 	}
 
-	gdk_pixbuf_unref (holes);
+	g_object_unref (holes);
 
 	return target;
 }
@@ -132,7 +140,7 @@ add_holes_to_pixbuf_large (GdkPixbuf *pixbuf, int size)
 
 	if (left == NULL)
 	{
-		gdk_pixbuf_ref (pixbuf);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
@@ -143,8 +151,8 @@ add_holes_to_pixbuf_large (GdkPixbuf *pixbuf, int size)
 
 	if (right == NULL)
 	{
-		gdk_pixbuf_unref (left);
-		gdk_pixbuf_ref (pixbuf);
+		g_object_unref (left);
+		g_object_ref (pixbuf);
 		return pixbuf;
 	}
 
@@ -279,7 +287,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 			with_holes = add_holes_to_pixbuf_small (small,
 					d_width, d_height);
 			g_return_if_fail (with_holes != NULL);
-			gdk_pixbuf_unref (small);
+			g_object_unref (small);
 		} else {
 			with_holes = small;
 		}
@@ -291,7 +299,8 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	a_width = g_strdup_printf ("%d", width);
 	a_height = g_strdup_printf ("%d", height);
 
-	if (gdk_pixbuf_save (with_holes, path, format, &err,
+	if (gdk_pixbuf_save (with_holes, path,
+				jpeg_output ? "jpeg" : "png", &err,
 				"tEXt::Thumb::Image::Width", a_width,
 				"tEXt::Thumb::Image::Height", a_height,
 				NULL) == FALSE)
@@ -307,7 +316,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 			g_print ("totem-video-thumbnailer couln't write the thumbnail '%s' for video '%s'\n", path, video_path);
 		}
 
-		gdk_pixbuf_unref (with_holes);
+		g_object_unref (with_holes);
 		return;
 	}
 
@@ -315,7 +324,7 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	show_pixbuf (with_holes);
 #endif
 
-	gdk_pixbuf_unref (with_holes);
+	g_object_unref (with_holes);
 }
 
 static gpointer
@@ -332,34 +341,44 @@ time_monitor (gpointer data)
 	exit (0);
 }
 
-static void
-usage (const char *cmd)
-{
-	g_print ("Usage: %s [-s <size>] [-j] <input> <output> [backend options]\n", cmd);
-}
+static const GOptionEntry entries[] = {
+	{ "jpeg", 'j',  0, G_OPTION_ARG_NONE, &jpeg_output, "Output the thumbnail as a JPEG instead of PNG", NULL },
+	{ "size", 's', 0, G_OPTION_ARG_INT, &output_size, "Size of the thumbnail in pixels", NULL },
+	{ "no-limit", 'l', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &time_limit, "Don't limit the thumbnailing time to 30 seconds", NULL },
+	{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, "Output debug information", NULL },
+	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames, "Movies to index", NULL },
+	{ NULL }
+};
 
 int main (int argc, char *argv[])
 {
+	GOptionGroup *options;
+	GOptionContext *context;
 	GError *err = NULL;
 	BaconVideoWidget *bvw;
 	GdkPixbuf *pixbuf;
-	int i, length, size;
+	int length;
 	char *input, *output;
 
-	const float frame_locations[] =
-	{ 1.0 / 3.0, 2.0 / 3.0, 0.1,0.9, 0.5 };
-	const int frame_locations_length = 5;
-	int current;
-
-	if (argc <= 2 || strcmp (argv[1], "-h") == 0 ||
-	    strcmp (argv[1], "--help") == 0) {
-		usage (argv[0]);
-		return -1;
-	}
+	const float frame_locations[] = {
+		1.0 / 3.0,
+		2.0 / 3.0,
+		0.1,
+		0.9,
+		0.5
+	};
+	guint current;
 
+#ifdef G_OS_UNIX
 	nice (20);
+#endif
 
 	g_thread_init (NULL);
+
+	context = g_option_context_new ("Thumbnail movies");
+	options = bacon_video_widget_get_option_group ();
+	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, options);
 #ifndef THUMB_DEBUG
 	g_type_init ();
 #else
@@ -370,32 +389,21 @@ int main (int argc, char *argv[])
 	gnome_authentication_manager_init ();
 #endif
 
-	bacon_video_widget_init_backend (&argc, &argv);
 	gnome_vfs_init ();
-
-	size = 128;
-	input = output = NULL;
-	i = 1;
-	while (i < argc) {
-		if (strcmp (argv[i], "-s") == 0) {
-			size = g_strtod (argv[++i], NULL);
-			i++;
-			continue;
-		}
-		if (strcmp (argv[i], "-j") == 0) {
-			format = "jpeg";
-			i++;
-			continue;
-		}
-		input = argv[i];
-		output = argv[++i];
-		break;
+	if (g_option_context_parse (context, &argc, &argv, &err) == FALSE) {
+		g_print ("couldn't parse command-line options: %s\n", err->message);
+		g_error_free (err);
+		return 1;
 	}
 
-	if (output == NULL || input == NULL) {
-		usage (argv[0]);
-		return -1;
+	if (filenames == NULL || g_strv_length (filenames) != 2) {
+		g_print ("Expects an input and an output file\n");
+		return 1;
 	}
+	input = filenames[0];
+	output = filenames[1];
+
+	PROGRESS_DEBUG("Initialised libraries, about to create video widget\n");
 
 	bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new (-1, -1, BVW_USE_TYPE_CAPTURE, &err));
 	if (err != NULL)
@@ -406,27 +414,15 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
+	PROGRESS_DEBUG("Video widget created\n");
+
+	if (time_limit != FALSE)
+		g_thread_create (time_monitor, (gpointer) input, FALSE, NULL);
+
+	PROGRESS_DEBUG("About to open video file\n");
 
 	if (bacon_video_widget_open (bvw, input, &err) == FALSE)
 	{
-		if (err->code == BVW_ERROR_STILL_IMAGE)
-		{
-			g_error_free (err);
-			err = NULL;
-			pixbuf = gdk_pixbuf_new_from_file (input, &err);
-			if (pixbuf != NULL)
-			{
-				save_pixbuf (pixbuf, output, input, size, TRUE);
-				gdk_pixbuf_unref (pixbuf);
-				exit (0);
-			}
-			g_print ("totem-video-thumbnailer couln't open file '%s'\n"
-					"Reason: %s.\n", input, err->message);
-			g_error_free (err);
-			exit (1);
-		}
-
 		g_print ("totem-video-thumbnailer couln't open file '%s'\n"
 				"Reason: %s.\n",
 				input, err->message);
@@ -434,23 +430,28 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
+	PROGRESS_DEBUG("Opened video file: '%s'\n", input);
+	PROGRESS_DEBUG("About to play file\n");
+
 	bacon_video_widget_play (bvw, &err);
 	if (err != NULL)
 	{
 		g_print ("totem-video-thumbnailer couln't play file: '%s'\n"
-				"Reason: %s.\n",
-				input, err->message);
+				"Reason: %s.\n", input, err->message);
 		g_error_free (err);
 		exit (1);
 	}
 
+	PROGRESS_DEBUG("Started playing file\n");
+
 	/* Test at multiple points in the file to see if we can get an 
 	 * interesting frame */
-	for (current = 0; current < frame_locations_length; current++)
+	for (current = 0; current < G_N_ELEMENTS(frame_locations); current++)
 	{
 		length = bacon_video_widget_get_stream_length (bvw);
 		if (length > MIN_LEN_FOR_SEEK)
 		{
+			PROGRESS_DEBUG("About to seek to %f\n", frame_locations[current]);
 			if (bacon_video_widget_seek
 					(bvw, frame_locations[current], NULL) == FALSE)
 			{
@@ -471,14 +472,20 @@ int main (int argc, char *argv[])
 		}
 
 		/* Pull the frame, if it's interesting we bail early */
+		PROGRESS_DEBUG("About to get frame for iter %d\n", current);
 		pixbuf = bacon_video_widget_get_current_frame (bvw);
-		if (is_image_interesting (pixbuf) != FALSE)
+		if (pixbuf != NULL && is_image_interesting (pixbuf) != FALSE) {
+			PROGRESS_DEBUG("Frame for iter %d is interesting\n", current);
 			break;
+		}
 
-		/* If we get to the end of this loop, we'll end up using the last image
-		 * we pulled */
-		if(current + 1 < frame_locations_length)
-			gdk_pixbuf_unref (pixbuf);
+		/* If we get to the end of this loop, we'll end up using
+		 * the last image we pulled */
+		if(current + 1 < G_N_ELEMENTS(frame_locations)) {
+			if (pixbuf != NULL)
+				g_object_unref (pixbuf);
+		}
+		PROGRESS_DEBUG("Frame for iter %d was not interesting\n", current);
 	}
 
 	/* Cleanup */
@@ -493,8 +500,9 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	save_pixbuf (pixbuf, output, input, size, FALSE);
-	gdk_pixbuf_unref (pixbuf);
+	PROGRESS_DEBUG("Saving captured screenshot\n");
+	save_pixbuf (pixbuf, output, input, output_size, FALSE);
+	g_object_unref (pixbuf);
 
 	return 0;
 }
